'
' Untested code to read an ADS 1015 or ADS1115 16-bit ADC.
' For Proton 24
'
    Device = 24FJ64GA002
    Declare Xtal = 32
'
' USART1 declares
'
    Declare Hserial_Baud = 9600             ' UART1 baud rate
    Declare Hrsout1_Pin = PORTB.3           ' Select which pin is to be used for TX with USART1
'
' Declare the Pins used for bit-bashed I2C communications
'
    Declare SDA_Pin = PORTB.0                   ' SDA connected to this pin
    Declare SCL_Pin = PORTB.1                   ' SCL connected to this pin
    'Declare Slow_Bus = On                      ' Slow down the interface (if required)

    Include "Bus.inc"                           ' Load the replacement Busin and Busout routines
'
' I2C Address/Bits
'
$define cADS_ADDRESS             $48            ' %1001000 (ADDR = GND)
'
' Conversion Delay (in mS)
'
$define cADS1015_CONVERSIONDELAY 1
$define cADS1115_CONVERSIONDELAY 8
'
' Pointer Register
'
$define cADS_POINTER_MASK        $03
$define cADS_POINTER_CONVERT     $00
$define cADS_POINTER_CONFIG      $01
$define cADS_POINTER_LOWTHRESH   $02
$define cADS_POINTER_HITHRESH    $03
'
' Config Register
'
$define cADS_OS_MASK      $8000
$define cADS_OS_SINGLE    $8000  ' Write: Set to start a single-conversion
$define cADS_OS_BUSY      $0000  ' Read: Bit = 0 when conversion is in progress
$define cADS_OS_NOTBUSY   $8000  ' Read: Bit = 1 when device is not performing a conversion

$define cADS_MUX_MASK     $7000
$define cADS_MUX_DIFF_0_1 $0000  ' Differential P = AIN0, N = AIN1 (default)
$define cADS_MUX_DIFF_0_3 $1000  ' Differential P = AIN0, N = AIN3
$define cADS_MUX_DIFF_1_3 $2000  ' Differential P = AIN1, N = AIN3
$define cADS_MUX_DIFF_2_3 $3000  ' Differential P = AIN2, N = AIN3
$define cADS_MUX_SINGLE_0 $4000  ' Single-ended AIN0
$define cADS_MUX_SINGLE_1 $5000  ' Single-ended AIN1
$define cADS_MUX_SINGLE_2 $6000  ' Single-ended AIN2
$define cADS_MUX_SINGLE_3 $7000  ' Single-ended AIN3

$define cADS_PGA_MASK     $0E00
$define cADS_PGA_6_144V   $0000  ' +/-6.144V range = Gain 2/3
$define cADS_PGA_4_096V   $0200  ' +/-4.096V range = Gain 1
$define cADS_PGA_2_048V   $0400  ' +/-2.048V range = Gain 2 (default)
$define cADS_PGA_1_024V   $0600  ' +/-1.024V range = Gain 4
$define cADS_PGA_0_512V   $0800  ' +/-0.512V range = Gain 8
$define cADS_PGA_0_256V   $0A00  ' +/-0.256V range = Gain 16

$define cADS_MODE_MASK    $0100
$define cADS_MODE_CONTIN  $0000  ' Continuous conversion mode
$define cADS_MODE_SINGLE  $0100  ' Power-down single-shot mode (default)

$define cADS_DR_MASK      $00E0
$define cADS_DR_128SPS    $0000  ' 128 samples per second
$define cADS_DR_250SPS    $0020  ' 250 samples per second
$define cADS_DR_490SPS    $0040  ' 490 samples per second
$define cADS_DR_920SPS    $0060  ' 920 samples per second
$define cADS_DR_1600SPS   $0080  ' 1600 samples per second (default)
$define cADS_DR_2400SPS   $00A0  ' 2400 samples per second
$define cADS_DR_3300SPS   $00C0  ' 3300 samples per second

$define cADS_CMODE_MASK   $0010
$define cADS_CMODE_TRAD   $0000  ' Traditional comparator with hysteresis (default)
$define cADS_CMODE_WINDOW $0010  ' Window comparator

$define cADS_CPOL_MASK    $0008
$define cADS_CPOL_ACTVLOW $0000  ' ALERT/RDY pin is low when active (default)
$define cADS_CPOL_ACTVHI  $0008  ' ALERT/RDY pin is high when active

$define cADS_CLAT_MASK    $0004  ' Determines if ALERT/RDY pin latches once asserted
$define cADS_CLAT_NONLAT  $0000  ' Non-latching comparator (default)
$define cADS_CLAT_LATCH   $0004  ' Latching comparator

$define cADS_CQUE_MASK    $0003
$define cADS_CQUE_1CONV   $0000  ' Assert ALERT/RDY after one conversions
$define cADS_CQUE_2CONV   $0001  ' Assert ALERT/RDY after two conversions
$define cADS_CQUE_4CONV   $0002  ' Assert ALERT/RDY after four conversions
$define cADS_CQUE_NONE    $0003  ' Disable the comparator and put ALERT/RDY in high state (default)
'
' Gain values
'
$define cGAIN_TWOTHIRDS    cADS_PGA_6_144V
$define cGAIN_ONE          cADS_PGA_4_096V
$define cGAIN_TWO          cADS_PGA_2_048V
$define cGAIN_FOUR         cADS_PGA_1_024V
$define cGAIN_EIGHT        cADS_PGA_0_512V
$define cGAIN_SIXTEEN      cADS_PGA_0_256V


    Dim Global_bADS_Address As Byte
    Dim Global_bADS_Delay As Byte
    Dim Global_bADS_BitShift As Byte
    Dim Global_wADS_Gain As Word


    Dim MyResult As Word

'---------------------------------------------------------------------------------------
' Writes 16-bits to the specified destination register
'
Proc ADS_WriteRegister(pI2CAddress As Byte, pReg As Byte, pValue As Word)
    BusOut pI2CAddress, pReg, [pValue.Byte1, pValue.Byte0]
EndProc

'---------------------------------------------------------------------------------------
' Writes 16-bits to the specified destination register
'
Proc ADS_ReadRegister(pI2CAddress As Byte, pReg As Byte), Word
    BusOut pI2CAddress,[cADS_POINTER_CONVERT]
    BusIn pI2CAddress, pReg, [Result.Byte1, Result.Byte0]
EndProc

'---------------------------------------------------------------------------------------
' Sets up for an ADS1015 ADC
'
Proc ADS1015_Setup(pI2CAddress As Byte)
    Global_bADS_Address = pI2CAddress
    Global_bADS_Delay = cADS1015_CONVERSIONDELAY
    Global_bADS_BitShift = 4
    Global_wADS_Gain = cGAIN_TWOTHIRDS      ' +/- 6.144V range (limited to VDD +0.3V max!)
EndProc

'---------------------------------------------------------------------------------------
' Sets up for an ADS1115 ADC
'
Proc ADS1115_Setup(pI2CAddress As Byte)
    Global_bADS_Address = pI2CAddress
    Global_bADS_Delay = cADS1115_CONVERSIONDELAY
    Global_bADS_BitShift = 0
    Global_wADS_Gain = cGAIN_TWOTHIRDS      ' +/- 6.144V range (limited to VDD +0.3V max!) *)
EndProc

'---------------------------------------------------------------------------------------
' Sets the gain and input voltage range
'
$define ADS_SetGain(pGain) Global_wADS_Gain = pGain

'---------------------------------------------------------------------------------------
' Gets a gain and input voltage range
'
$define ADS_GetGain() Global_wADS_Gain

'---------------------------------------------------------------------------------------
' Gets a single-ended ADC reading from the specified channel
'
Proc ADS_ReadADC_SingleEnded(pChannel As Byte), Word
' Start with default values
'
    Dim wConfig As Word
    wConfig = cADS_CQUE_NONE | cADS_CLAT_NONLAT | cADS_CPOL_ACTVLOW | cADS_CMODE_TRAD | cADS_DR_1600SPS | cADS_MODE_SINGLE  ' Disable the comparator (default)
                                                                                                                            ' Non-latching (default)
                                                                                                                            ' Alert/Rdy active low (default)
                                                                                                                            ' Traditional comparator (default)
                                                                                                                            ' 1600 samples per second (default)
                                                                                                                            ' Single-shot mode (default)
' Set PGA/voltage range
'
    wConfig = wConfig | Global_wADS_Gain
'
' Set single-ended input channel
'
    If pChannel = 0 Then
        wConfig = wConfig | cADS_MUX_SINGLE_0
    ElseIf pChannel = 1 Then
        wConfig = wConfig | cADS_MUX_SINGLE_1
    ElseIf pChannel = 2 Then
        wConfig = wConfig | cADS_MUX_SINGLE_2
    ElseIf pChannel = 3 Then
        wConfig = wConfig | cADS_MUX_SINGLE_3
    Else
        Result = 0
        Return
    EndIf
'
' Set start single-conversion bit
'
    wConfig = wConfig | cADS_OS_SINGLE
'
' Write wConfig register to the ADC
'
    ADS_WriteRegister(Global_bADS_Address, cADS_POINTER_CONFIG, wConfig)
'
' Wait for the conversion to complete
'
    DelayMS Global_bADS_Delay
'
' Read the conversion results
' Shift 12-bit results right 4 bits for the ADS1015
'
    Result = ADS_ReadRegister(Global_bADS_Address, cADS_POINTER_CONVERT)
    Result = Result >> Global_bADS_BitShift
EndProc

'---------------------------------------------------------------------------------------
' Reads the conversion results, measuring the voltage
' difference between the P (AIN0) and N (AIN1) input.
' Generates a signed value since the difference can be either positive or negative.
'
Proc ADS_ReadADC_Differential_0_1(), SWord
' Start with default values
'
    Dim wRes As Word
    Dim wConfig As Word
    wConfig = cADS_CQUE_NONE | cADS_CLAT_NONLAT | cADS_CPOL_ACTVLOW | cADS_CMODE_TRAD | cADS_DR_1600SPS | cADS_MODE_SINGLE  ' Disable the comparator (default)
                                                                                                                            ' Non-latching (default)
                                                                                                                            ' Alert/Rdy active low (default)
                                                                                                                            ' Traditional comparator (default)
                                                                                                                            ' 1600 samples per second (default)
                                                                                                                            ' Single-shot mode (default)
' Set PGA/voltage range
'
    wConfig = wConfig | Global_wADS_Gain
'
' Set channels
'
    wConfig = wConfig | cADS_MUX_DIFF_0_1          ' AIN0 = P, AIN1 = N
'
' Set 'start single-conversion' bit
'
    wConfig = wConfig | cADS_OS_SINGLE
'
' Write config register to the ADC
'
    ADS_WriteRegister(Global_bADS_Address, cADS_POINTER_CONFIG, wConfig)
'
' Wait for the conversion to complete
'
    DelayMS Global_bADS_Delay
'
' Read the conversion results
'
    wRes = ADS_ReadRegister(Global_bADS_Address, cADS_POINTER_CONVERT)
    wRes = wRes >> Global_bADS_BitShift
    If Global_bADS_BitShift > 0 Then
        ' Shift 12-bit results right 4 bits for the ADS1015, making sure to keep the sign bit intact
        If wRes > $07FF Then
            wRes = wRes | $F000         ' Negative number - extend the sign to 16th bit
        EndIf
    EndIf
    Result = wRes
EndProc

'---------------------------------------------------------------------------------------
' Reads the conversion results, measuring the voltage
' difference between the P (AIN2) and N (AIN3) input.
' Generates a signed value since the difference can be either positive or negative.
'
Proc ADS_ReadADC_Differential_2_3(), SWord
' Start with default values
    Dim wRes As Word
    Dim wConfig As Word
    wConfig = cADS_CQUE_NONE | cADS_CLAT_NONLAT | cADS_CPOL_ACTVLOW | cADS_CMODE_TRAD | cADS_DR_1600SPS | cADS_MODE_SINGLE  ' Disable the comparator (default)
                                                                                                                            ' Non-latching (default)
                                                                                                                            ' Alert/Rdy active low (default)
                                                                                                                            ' Traditional comparator (default)
                                                                                                                            ' 1600 samples per second (default)
                                                                                                                            ' Single-shot mode (default)
' Set PGA/voltage range
'
    wConfig = wConfig | Global_wADS_Gain
'
' Set channels
'
    wConfig = wConfig | cADS_MUX_DIFF_2_3          ' AIN2 = P, AIN3 = N
'
' Set 'start single-conversion' bit
'
    wConfig = wConfig | cADS_OS_SINGLE
'
' Write config register to the ADC
'
    ADS_WriteRegister(Global_bADS_Address, cADS_POINTER_CONFIG, wConfig)

' Wait for the conversion to complete
    DelayMS Global_bADS_Delay
'
' Read the conversion results
'
    wRes = ADS_ReadRegister(Global_bADS_Address, cADS_POINTER_CONVERT)
    wRes = wRes >> Global_bADS_BitShift
    If Global_bADS_BitShift > 0 Then
        '
        ' Shift 12-bit results right 4 bits for the ADS1015, making sure to keep the sign bit intact
        '
        If wRes > $07FF Then
            wRes = wRes | $F000             ' Negative number - extend the sign to 16th bit
        EndIf
    EndIf
    Result = wRes
EndProc

'---------------------------------------------------------------------------------------
' Sets up the comparator to operate in basic mode, causing the
'            ALERT/RDY pin to assert (go from high to low) when the ADC
'            value exceeds the specified threshold.
'            This will also set the ADC in continuous conversion mode.
'
Proc ADS_StartComparator_SingleEnded(pChannel As Byte, pThreshold As SWord)
' Start with default values
'
    Dim wConfig As Word
    wConfig = cADS_CQUE_1CONV | cADS_CLAT_LATCH | cADS_CPOL_ACTVLOW | cADS_CMODE_TRAD | cADS_DR_1600SPS | cADS_MODE_CONTIN  ' Comparator enabled and asserts on 1 match
                                                                                                                            ' Latching mode
                                                                                                                            ' Alert/Rdy active low (default)
                                                                                                                            ' Traditional comparator (default)
                                                                                                                            ' 1600 samples per second (default)
                                                                                                                            ' Continuous conversion mode
' Set PGA/voltage range
'
    wConfig = wConfig | Global_wADS_Gain
'
' Set single-ended input channel
'
    If pChannel = 0 Then
        wConfig = wConfig | cADS_MUX_SINGLE_0
    ElseIf pChannel = 1 Then
        wConfig = wConfig | cADS_MUX_SINGLE_1
    ElseIf pChannel = 2 Then
        wConfig = wConfig | cADS_MUX_SINGLE_2
    ElseIf pChannel = 3 Then
        wConfig = wConfig | cADS_MUX_SINGLE_3
    Else
        Result = 0
        Return
    EndIf
'
' Set the high threshold register
' Shift 12-bit results left 4 bits for the ADS1015
'
    ADS_WriteRegister(Global_bADS_Address, cADS_POINTER_HITHRESH, pThreshold << Global_bADS_BitShift)
'
' Write config register to the ADC
'
    ADS_WriteRegister(Global_bADS_Address, cADS_POINTER_CONFIG, wConfig)
EndProc

'---------------------------------------------------------------------------------------
' In order to clear the comparator, we need to read the
'            conversion results.  This function reads the last conversion
'            results without changing the config value.
'
Proc ADS_GetLastConversionResults(), SWord
    Dim wRes As Word
'
' Wait for the conversion to complete
'
    DelayMS Global_bADS_Delay
'
' Read the conversion results
'
    wRes = ADS_ReadRegister(Global_bADS_Address, cADS_POINTER_CONVERT)
    wRes = wRes  >> Global_bADS_BitShift
    If Global_bADS_BitShift > 0 Then
        '
        ' Shift 12-bit results right 4 bits for the ADS1015, making sure to keep the sign bit intact
        '
        If wRes > $07FF Then
            wRes = wRes | $F000             ' Negative number - extend the sign to 16th bit
        EndIf
    EndIf
    Result = wRes
EndProc
'
'-------------------------------------------------------------------------------------------------------------
' The main program loop starts here
'
Main:
    CLKDIV = 0                                              ' \
    Write_OSCCONH(%00010000)                                ' / Enable the PLL to operate the device at 32MHz

    PPS_Output(cOut_Pin_RP3, cOut_Fn_U1TX)                  ' Make PPS Pin RP3 U1TX

    ADS1115_Setup(cADS_ADDRESS)                             ' Set up for an ADS1115 with a default I2C address of 0
    While
        MyResult = ADS_ReadADC_SingleEnded(0)
        HRSOut "Single_Result = ", Dec MyResult, 13
        MyResult = ADS_ReadADC_Differential_0_1()
        HRSOut "Mult_0_1_Result = ", SDec MyResult, 13
        MyResult = ADS_ReadADC_Differential_2_3()
        HRSOut "Mult_2_3_Result = ", Dec MyResult, 13
    Wend


'-------------------------------------------------------------------------------------------------------------
' For internal 8MHz oscillator with PLL
' OSC pins are general purpose I/O
'
    Config Config1 = JTAGEN_OFF, GCP_OFF, BKBUG_OFF, COE_OFF, ICS_PGx1, FWDTEN_OFF, WINDIS_OFF, FWPSA_PR128, WDTPOST_PS256
    Config Config2 = IOL1WAY_OFF, COE_OFF, IESO_OFF, FNOSC_FRCPLL, FCKSM_CSECME, OSCIOFNC_ON, POSCMOD_NONE

